<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Class WAEF_Match_Conditions.
 *
 * The WAEF Match Conditions class handles the matching rules for fees.
 *
 * @class		WAEF_Match_Conditions
 * @author		multisquares
 * @package 	WooCommerce Advanced Extra Fees Lite
 * @version	1.0.0
 */
class WAEF_Match_Conditions {


	/**
	 * Constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		add_filter( 'waef_match_condition_subtotal', array( $this, 'waef_match_condition_subtotal' ), 10, 4 );
		add_filter( 'waef_match_condition_contains_product', array( $this, 'waef_match_condition_contains_product' ), 10, 4 );
		
		add_filter( 'waef_match_condition_country', array( $this, 'waef_match_condition_country' ), 10, 4 );
		add_filter( 'waef_match_condition_stock_status', array( $this, 'waef_match_condition_stock_status' ), 10, 4 );
		add_filter( 'waef_match_condition_category', array( $this, 'waef_match_condition_category' ), 10, 4 );

	}


	/**
	 * Subtotal.
	 *
	 * Match the condition value against the cart subtotal.
	 *
	 * @since 1.0.0
	 *
	 * @param  bool   $match    Current match value.
	 * @param  string $operator Operator selected by the user in the condition row.
	 * @param  mixed  $value    Value given by the user in the condition row.
	 * @param  array  $package  List of shipping package details.
	 * @return BOOL             Matching result, TRUE if results match, otherwise FALSE.
	 */
	public function waef_match_condition_subtotal( $match, $operator, $value, $package ) {

		if ( ! isset( WC()->cart ) ) :
			return $match;
		endif;

		// Make sure its formatted correct
		$value = str_replace( ',', '.', $value );

		// WPML multi-currency support
		$value = apply_filters( 'wcml_shipping_price_amount', $value );

		$subtotal = WC()->cart->subtotal;

		if ( '==' == $operator ) :
			$match = ( $subtotal == $value );
		elseif ( '!=' == $operator ) :
			$match = ( $subtotal != $value );
		elseif ( '>=' == $operator ) :
			$match = ( $subtotal >= $value );
		elseif ( '<=' == $operator ) :
			$match = ( $subtotal <= $value );
		endif;

		return $match;

	}


	/**
	 * Contains product.
	 *
	 * Matches if the condition value product is in the cart.
	 *
	 * @since 1.0.0
	 *
	 * @param  bool   $match    Current match value.
	 * @param  string $operator Operator selected by the user in the condition row.
	 * @param  mixed  $value    Value given by the user in the condition row.
	 * @param  array  $package  List of shipping package details.
	 * @return BOOL             Matching result, TRUE if results match, otherwise FALSE.
	 */
	public function waef_match_condition_contains_product( $match, $operator, $value, $package ) {

		foreach ( $package->cart_contents as $product ) :
			$product_ids[] = $product['product_id'];
		endforeach;

		if ( '==' == $operator ) :
			$match = ( in_array( $value, $product_ids ) );
		elseif ( '!=' == $operator ) :
			$match = ( ! in_array( $value, $product_ids ) );
		endif;

		return $match;

	}




/******************************************************
 * User conditions
 *****************************************************/

	/**
	 * Country.
	 *
	 * Match the condition value against the users shipping country.
	 *
	 * @since 1.0.0
	 *
	 * @param  bool   $match    Current match value.
	 * @param  string $operator Operator selected by the user in the condition row.
	 * @param  mixed  $value    Value given by the user in the condition row.
	 * @param  array  $package  List of shipping package details.
	 * @return BOOL             Matching result, TRUE if results match, otherwise FALSE.
	 */
	public function waef_match_condition_country( $match, $operator, $value, $package ) {

		if ( ! isset( WC()->customer ) ) :
			return $match;
		endif;

		if ( '==' == $operator ) :
			$match = ( preg_match( '/^' . preg_quote( $value, '/' ) . "$/i", WC()->customer->get_shipping_country() ) );
		elseif ( '!=' == $operator ) :
			$match = ( ! preg_match( '/^' . preg_quote( $value, '/' ) . "$/i", WC()->customer->get_shipping_country() ) );
		endif;

		return $match;

	}




/******************************************************
 * Product conditions
 *****************************************************/

	/**
	 * Stock status.
	 *
	 * Match the condition value against all cart products stock statuses.
	 *
	 * @since 1.0.0
	 *
	 * @param  bool   $match    Current match value.
	 * @param  string $operator Operator selected by the user in the condition row.
	 * @param  mixed  $value    Value given by the user in the condition row.
	 * @param  array  $package  List of shipping package details.
	 * @return BOOL             Matching result, TRUE if results match, otherwise FALSE.
	 */
	public function waef_match_condition_stock_status( $match, $operator, $value, $package ) {

		if ( '==' == $operator ) :

			$match = true;
			foreach ( $package->cart_contents as $product ) :

				if ( true == $product['data']->variation_has_stock ) :
					$stock_status = ( get_post_meta( $product['data']->variation_id, '_stock_status', true ) );
				else :
					$stock_status = ( get_post_meta( $product['product_id'], '_stock_status', true ) );
				endif;

				if ( $stock_status != $value ) :
					$match = false;
				endif;

			endforeach;

		elseif ( '!=' == $operator ) :

			$match = true;
			foreach ( $package->cart_contents as $product ) :

				if ( true == $product['data']->variation_has_stock ) :
					$stock_status = ( get_post_meta( $product['data']->variation_id, '_stock_status', true ) );
				else :
					$stock_status = ( get_post_meta( $product['product_id'], '_stock_status', true ) );
				endif;

				if ( $stock_status == $value ) :
					$match = false;
				endif;

			endforeach;

		endif;

		return $match;

	}


	/**
	 * Category.
	 *
	 * Match the condition value against all the cart products category.
	 * With this condition, all the products in the cart must have the given class.
	 *
	 * @since 1.0.0
	 *
	 * @param  bool   $match    Current match value.
	 * @param  string $operator Operator selected by the user in the condition row.
	 * @param  mixed  $value    Value given by the user in the condition row.
	 * @param  array  $package  List of shipping package details.
	 * @return BOOL             Matching result, TRUE if results match, otherwise FALSE.
	 */
	public function waef_match_condition_category( $match, $operator, $value, $package ) {

		if ( ! isset( WC()->cart ) ) :
			return $match;
		endif;

		$match = true;

		if ( '==' == $operator ) :

			foreach ( WC()->cart->cart_contents as $product ) :

				if ( ! has_term( $value, 'product_cat', $product['product_id'] ) ) :
					$match = false;
				endif;

			endforeach;

		elseif ( '!=' == $operator ) :

			foreach ( WC()->cart->cart_contents as $product ) :

				if ( has_term( $value, 'product_cat', $product['product_id'] ) ) :
					$match = false;
				endif;

			endforeach;

		endif;

		return $match;

	}


}
